//**************************************************************************
//**
//**	##   ##    ##    ##   ##   ####     ####   ###     ###
//**	##   ##  ##  ##  ##   ##  ##  ##   ##  ##  ####   ####
//**	 ## ##  ##    ##  ## ##  ##    ## ##    ## ## ## ## ##
//**	 ## ##  ########  ## ##  ##    ## ##    ## ##  ###  ##
//**	  ###   ##    ##   ###    ##  ##   ##  ##  ##       ##
//**	   #    ##    ##    #      ####     ####   ##       ##
//**
//**	Copyright (C) 1999-2000 Jnis Legzdi
//**
//**	This program is free software; you can redistribute it and/or
//**  modify it under the terms of the GNU General Public License
//**  as published by the Free Software Foundation; either version 2
//**  of the License, or (at your option) any later version.
//**
//**	This program is distributed in the hope that it will be useful,
//**  but WITHOUT ANY WARRANTY; without even the implied warranty of
//**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//**  GNU General Public License for more details.
//**	
//**************************************************************************

//===========================================================================
// Bat Spawner Variables
//	special1	frequency counter
//	special2	
//	args[0]		frequency of spawn (1=fastest, 10=slowest)
//	args[1]		spread angle (0..255)
//	args[2]		
//	args[3]		duration of bats (in octics)
//	args[4]		turn amount per move (in degrees)
//
// Bat Variables
//	special2	lifetime counter
//	args[4]		turn amount per move (in degrees)
//===========================================================================

// MACROS ------------------------------------------------------------------

// CONSTANTS ---------------------------------------------------------------

// TYPES -------------------------------------------------------------------

// FUNCTION PROTOTYPES -----------------------------------------------------

// DATA DECLARATIONS -------------------------------------------------------

// CODE --------------------------------------------------------------------

//===========================================================================
//
//  A_BatSpawnInit
//
//===========================================================================

void A_BatSpawnInit(mobj_t* actor)
{
	actor->special1 = 0;	// Frequency count
}

//===========================================================================
//
//  A_BatSpawn
//
//===========================================================================

void A_BatSpawn(mobj_t* actor)
{
	mobj_t*	mo;
	int 	delta;
	int		angle;

	// Countdown until next spawn
	if (actor->special1-- > 0) return;
	actor->special1 = actor->args[0];		// Reset frequency count

	delta = actor->args[1];
	if (delta == 0) delta = 1;
	angle = actor->angles.yaw + (((P_Random() % delta) - (delta >> 1)) << 24);
	mo = SpawnMissileAngle(actor, MT_BAT, angle, 0.0);
	if (mo)
	{
		mo->args[0] = P_Random() & 63;			// floatbob index
		mo->args[4] = actor->args[4];			// turn degrees
		mo->special2 = actor->args[3] << 3;		// Set lifetime
		mo->target = actor;
	}
}

//===========================================================================
//
//  A_BatMove
//
//===========================================================================

void A_BatMove(mobj_t* actor)
{
	int		newangle;
	float	speed;

	if (actor->special2 < 0)
	{
		SetMobjState(actor, actor->deathstate);
	}
	actor->special2 -= 2;		// Called every 2 tics

	if (P_Random()<128)
	{
		newangle = actor->angles.yaw + ANGLE_1 * actor->args[4];
	}
	else
	{
		newangle = actor->angles.yaw - ANGLE_1 * actor->args[4];
	}

	// Adjust momentum vector to new direction
	speed = actor->speed * Random() * 4.0;
	actor->velocity.x = speed * cos(newangle);
	actor->velocity.y = speed * sin(newangle);

	if (P_Random() < 15)
		StartSoundName(actor, "BatScream");

	// Handle Z movement
	actor->origin.z = actor->target->origin.z + 2.0 * FloatBobOffsets[actor->args[0]];
	actor->args[0] = (actor->args[0] + 3) & 63;
}

